<?php

/**
 * Carerix Wordpress Plugin
 *
 * @category template manager
 * @author Bogdan Ivanov
 * @copyright Copyright (c) 2013 Carerix.com (http://www.carerix.com)
 * @link http://www.carerix.com
 * @version $Rev$
 */
class Admin_Taxonomies_Slugger extends WP_Page {

    protected $interfacer = null;

    public function getCapability() {
        return WP_Constants::CAPABILITY_ADMIN;
    }

    public function getMenuSlug() {
        return 'carerix_taxonomies_slugger_page';
    }

    public function getMenuTitle() {
        return 'Slugs';
    }

    public function getPageTitle() {
        return 'Slugs';
    }

    public function page() {
        unset($_POST['submit']);

        global $carerix_app;

        $input_data = filter_input_array(INPUT_POST, FILTER_SANITIZE_STRING);

        if (is_array($input_data) && wp_verify_nonce($input_data['_carerix_nonce_field'], 'taxonomies')) {
            foreach ($input_data as $key => $value) {
                Environment_Settings::_set_setting_unique($key, $value);
            }
            if (count($input_data) > 0) {
                global $carerix_app;
                $carerix_app->reset();
            }
        }

        $data = array();
        $languages = array_keys((false === $wpml = $this->interfacer->get_wpml_active_languages() ) ? array() : $wpml );
        $currentLanguage = $this->interfacer->language_major_current();

        if ($languages != FALSE) {
            $data['headerLanguages'] = $languages;
        } else {
            $data['headerLanguages'] = array($currentLanguage);
        }

        $labelKeys = array(
            '1|general.message.field.country' => 'publications',
            '2|general.message.field.region' => 'publications',
            '3|general.header.education' => 'publications',
            '4|CxPortal.header.functionGroups' => 'publications',
            '5|CxPortal.message.field.workLocation' => 'publications',
            '5.1|CxPortal.message.field.procedure' => 'publications',
            '6|general.message.field.country' => 'candidate',
            '7|general.message.field.region' => 'candidate',
            '8|general.header.educationLevel_1' => 'candidate',
            '9|general.header.educationLevel_2' => 'candidate',
            '10|general.header.educationLevel_3' => 'candidate',
            '11|CxPortal.message.field.professionalLevel' => 'publications'
        );

        $rows = array();
        $slugs = array(
            '1|general.message.field.country' => WP_Constants::TAXONOMY_COUNTRY_SLUG,
            '2|general.message.field.region' => WP_Constants::TAXONOMY_REGION_SLUG,
            '3|general.header.education' => WP_Constants::TAXONOMY_EDUCATION_SLUG,
            '4|CxPortal.header.functionGroups' => WP_Constants::TAXONOMY_FUNCTION_SLUG,
            '5|CxPortal.message.field.workLocation' => WP_Constants::TAXONOMY_WORKLOCATION_SLUG,
            '5.1|CxPortal.message.field.procedure' => WP_Constants::TAXONOMY_PROCEDURE_SLUG,
            '6|general.message.field.country' => WP_Constants::TAXONOMY_CANDIDATE_LAND_SLUG,
            '7|general.message.field.region' => WP_Constants::TAXONOMY_CANDIDATE_REGION_SLUG,
            '8|general.header.educationLevel_1' => WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_1_SLUG,
            '9|general.header.educationLevel_2' => WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_2_SLUG,
            '10|general.header.educationLevel_3' => WP_Constants::TAXONOMY_CANDIDATE_EDUCATION_LEVEL_3_SLUG,
            '11|CxPortal.message.field.professionalLevel' => WP_Constants::TAXONOMY_WORKLEVEL_SLUG,
        );
        $labels = Utils_Labels::get();
        foreach ($labelKeys as $labelKey => $section) {
            $old_label_key = $labelKey;
            $_t = explode('|', $labelKey);
            $labelKey = $_t[1];
            $extra = '';
            $slugKey = $labelKey;

            if (stripos($labelKey, '_') !== FALSE) {
                $_labelKey = explode('_', $labelKey);
                $slugKey = $_labelKey[0];
                $extra = $_labelKey[1];
            }

            $fields = array();
            $name = "slug_taxonomy_{$section}_" . $slugs[$old_label_key];
            $fields['label'] = $labels[$slugKey] . ' ' . $extra;
            $fields['section'] = $section;
            $fields['inputs'] = array();

            foreach ($data['headerLanguages'] as $language) {
                $key = $name . '_' . $language;
                $key = str_replace('.', '_', $key);
                $value = Environment_Settings::_setting_get($key);
                if (!$value)
                    $value = $slugs[$old_label_key];

                $input = array('name' => $key, 'value' => $value);
                array_push($fields['inputs'], $input);
            }
            array_push($rows, $fields);
        }
        $data['rows'] = $rows;
        $template = new Utils_Template('admin/taxonomies/slugger');
        $template->with($data)->display();
    }

}
